"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

Object subclass:#GDBPTY
	instanceVariableNames:'name master slave'
	classVariableNames:''
	poolDictionaries:'TTYConstants'
	category:'GDB-Internal'
!

!GDBPTY class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!GDBPTY class methodsFor:'instance creation'!

new
    "return an initialized instance"

    self assert: OperatingSystem isUNIXlike.
    ^ self basicNew initialize.

    "Modified: / 11-01-2018 / 22:59:14 / jv"
! !

!GDBPTY methodsFor:'accessing'!

master
    ^ master
!

name
    ^ name
!

slave
    ^ slave
! !

!GDBPTY methodsFor:'initialization & release'!

initialize
    | triplet |

    OperatingSystem isUNIXlike ifTrue:[ 
        "Use a pseudo-tty"
        triplet := OperatingSystem makePTY.

        name := triplet at:3.

        "/ pty at:1 is the master;
        "/ pty at:2 is the slave
        master := NonPositionableExternalStream forReadWriteToFileDescriptor:(triplet at:1).
        master buffered:false.

        slave := NonPositionableExternalStream forReadWriteToFileDescriptor:(triplet at:2).
        slave buffered:false.
    ] ifFalse:[ 
        OperatingSystem isMSWINDOWSlike ifTrue:[ 
            self error: 'Windows are not (yet) supported'
        ] ifFalse:[ 
            self error: 'Operating system not supported'
        ]
    ].

    "Modified: / 31-05-2017 / 20:14:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 12-07-2023 / 12:16:43 / Jan Vrany <jan.vrany@labware.com>"
!

release
    name := nil.
    master notNil ifTrue:[ 
        master close.
        master := nil.
    ].
    slave notNil ifTrue:[ 
        slave close.
        slave := nil.
    ].

    "Created: / 09-06-2014 / 18:25:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBPTY methodsFor:'printing & storing'!

printOn:aStream
    "append a printed representation of the receiver to the argument, aStream"

    super printOn:aStream.
    aStream nextPutAll:'('.
    name printOn:aStream.
    aStream nextPutAll:')'.

    "Modified: / 31-05-2017 / 20:13:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBPTY methodsFor:'setup'!

setLocalEcho: aBoolean
    | attrs |

    attrs := master getTTYAttributes.
    aBoolean ifTrue:[ 
        attrs c_lflag: (attrs c_lflag bitOr: ECHO).
    ] ifFalse:[ 
        attrs c_lflag: (attrs c_lflag bitClear: ECHO).
    ].
    master setTTYAttributes: attrs.

    "Created: / 31-05-2017 / 20:19:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setOutputCRLF: aBoolean
    | attrs |

    attrs := master getTTYAttributes.
    aBoolean ifTrue:[ 
        attrs c_oflag: (attrs c_oflag bitOr: ONLCR).
    ] ifFalse:[ 
        attrs c_oflag: (attrs c_oflag bitClear: ONLCR).
    ].
    master setTTYAttributes: attrs.

    "Created: / 31-05-2017 / 20:20:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBPTY class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

